# -*- coding: utf-8 -*-

"""
Module for API-related calls.
"""

try:
    from urllib import parse as urlparse
except ImportError:
    import urlparse


def hostname_tld_migration(hostname):
    """
    Migrate transifex.net to transifex.com.

    :param hostname: The hostname to migrate (if needed).
    :returns: A hostname with the transifex.com domain (if needed).
    """
    parts = urlparse.urlparse(hostname)
    if parts.hostname.endswith('transifex.net'):
        hostname = hostname.replace('transifex.net', 'transifex.com', 1)
    return hostname


def hostname_ssl_migration(hostname):
    """
    Migrate Transifex hostnames to use HTTPS.

    :param hostname: The hostname to migrate (if needed).
    :returns: A https hostname (if needed).
    """
    parts = urlparse.urlparse(hostname)
    is_transifex = (
        parts.hostname[-14:-3] == '.transifex.' or
        parts.hostname == 'transifex.net' or
        parts.hostname == 'transifex.com'
    )
    is_https = parts.scheme == 'https'
    if is_transifex and not is_https:
        if not parts.scheme:
            hostname = 'https:' + hostname
        else:
            hostname = hostname.replace(parts.scheme, 'https', 1)
    return hostname


def visit_hostname(hostname):
    """
    Have a chance to visit a hostname before actually using it.

    :param hostname: The original hostname.
    :returns: The hostname with the necessary changes.
    """
    for processor in [hostname_ssl_migration, hostname_tld_migration, ]:
        hostname = processor(hostname)
    return hostname



# These are the Transifex API urls
# flake8: noqa

API_URLS = {
    'get_resources': '/api/2/project/%(project)s/resources/',
    'project_details': '/api/2/project/%(project)s/?details',
    'resource_details': '/api/2/project/%(project)s/resource/%(resource)s/',
    'pull_file': '/api/2/project/%(project)s/resource/%(resource)s/translation/%(language)s/?file',
    'pull_pseudo_file': '/api/2/project/%(project)s/resource/%(resource)s/pseudo/?pseudo_type=MIXED',
    'pull_reviewed_file': '/api/2/project/%(project)s/resource/%(resource)s/translation/%(language)s/?file&mode=reviewed',
    'pull_reviewed1_file': '/api/2/project/%(project)s/resource/%(resource)s/translation/%(language)s/?file&mode=reviewed1',
    'pull_reviewed2_file': '/api/2/project/%(project)s/resource/%(resource)s/translation/%(language)s/?file&mode=reviewed2',
    'pull_finalized_file': '/api/2/project/%(project)s/resource/%(resource)s/translation/%(language)s/?file&mode=finalized',
    'pull_sourceastranslation_file': '/api/2/project/%(project)s/resource/%(resource)s/translation/%(language)s/?file&mode=sourceastranslation',
    'pull_translator_file': '/api/2/project/%(project)s/resource/%(resource)s/translation/%(language)s/?file&mode=translator',
    'pull_onlytranslated_file': '/api/2/project/%(project)s/resource/%(resource)s/translation/%(language)s/?file&mode=onlytranslated',
    'pull_onlyreviewed_file': '/api/2/project/%(project)s/resource/%(resource)s/translation/%(language)s/?file&mode=onlyreviewed',
    'pull_onlyreviewed1_file': '/api/2/project/%(project)s/resource/%(resource)s/translation/%(language)s/?file&mode=onlyreviewed1',
    'pull_onlyreviewed2_file': '/api/2/project/%(project)s/resource/%(resource)s/translation/%(language)s/?file&mode=onlyreviewed2',
    'pull_onlyfinalized_file': '/api/2/project/%(project)s/resource/%(resource)s/translation/%(language)s/?file&mode=onlyfinalized',
    'pull_developer_file': '/api/2/project/%(project)s/resource/%(resource)s/translation/%(language)s/?file&mode=default',
    'resource_stats': '/api/2/project/%(project)s/resource/%(resource)s/stats/',
    'create_resource': '/api/2/project/%(project)s/resources/',
    'push_source': '/api/2/project/%(project)s/resource/%(resource)s/content/',
    'push_translation': '/api/2/project/%(project)s/resource/%(resource)s/translation/%(language)s/',
    'delete_translation': '/api/2/project/%(project)s/resource/%(resource)s/translation/%(language)s/',
    'formats': '/api/2/formats/',
    'delete_resource': '/api/2/project/%(project)s/resource/%(resource)s/',
    'organizations': '/organizations/',
    'projects': '/organizations/%(organization)s/projects/',
    'auth_check': '/auth_check/',
}

HOSTNAMES = {
    'hostname': ['formats'],
    'api_hostname': ['auth_check', 'projects', 'organizations'],
}



# -*- coding: utf-8 -*-

import unittest
from mock import patch, MagicMock

from six import assertRaisesRegex

from txclib.wizard import Wizard


@patch('txclib.wizard.Api')
class WizardCase(unittest.TestCase):
    def setUp(self, *args, **kwargs):
        with patch('txclib.wizard.Project') as project_mock:
            with patch('txclib.wizard.Api'):
                instance_mock = MagicMock()
                instance_mock.getset_host_credentials.return_value = \
                    'foo', 'bar'
                project_mock.return_value = instance_mock
                self.wizard = Wizard('/foo/bar/tmp')
        super(WizardCase, self).setUp(*args, **kwargs)

    def test_get_organizations(self, api_mock):
        self.wizard.api.get.return_value = [
            {
                "name": "Transifex",
                "slug": "transifex",
            },
            {
                "name": "Dunder Mifflin",
                "slug": "dunder-mifflin",
            }
        ]
        orgs = self.wizard.get_organizations()
        self.assertEqual(
            orgs,
            [("dunder-mifflin", "Dunder Mifflin"), ("transifex", "Transifex")]
        )

    def test_get_projects_excludes_archived(self, api_mock):
        self.wizard.api.get.return_value = [
            {
                "name": "project 1",
                "slug": "project-1",
                "archived": False
            },
            {
                "name": "project 2",
                "slug": "project-2",
                "archived": True
            }
        ]
        projects = self.wizard.get_projects_for_org('org_slug')
        self.assertEqual(
            projects,
            [
                {
                    "archived": False,
                    "name": "project 1",
                    "slug": "project-1"
                }
            ]
        )

    def test_get_formats_with_extension(self, api_mock):
        self.wizard.api.get.return_value = {
            "INI": {
                "mimetype": "text/plain",
                "file-extensions": ".ini",
                "description": "Joomla INI File"
            },
            "SRT": {
                "mimetype": "text/plain",
                "file-extensions": ".srt",
                "description": "SubRip subtitles"
            }
        }
        formats = self.wizard.get_formats('test.srt')
        self.assertEqual(formats, [("SRT", "SubRip subtitles - .srt")])

    def test_get_formats_without_extension(self, api_mock):
        self.wizard.api.get.return_value = {
            "INI": {
                "mimetype": "text/plain",
                "file-extensions": ".ini",
                "description": "Joomla INI File"
            },
            "SRT": {
                "mimetype": "text/plain",
                "file-extensions": ".srt",
                "description": "SubRip subtitles"
            }
        }
        formats = self.wizard.get_formats('test')
        self.assertEqual(
            formats,
            [("INI", "Joomla INI File - .ini"),
             ("SRT", "SubRip subtitles - .srt")]
        )

    def test_unsupported_formats(self, api_mock):
        self.wizard.api.get.return_value = {
            "INI": {
                "mimetype": "text/plain",
                "file-extensions": ".ini",
                "description": "Joomla INI File"
            },
            "SRT": {
                "mimetype": "text/plain",
                "file-extensions": ".srt",
                "description": "SubRip subtitles"
            }
        }
        error_msg = "No formats found for this file"
        with assertRaisesRegex(self, Exception, error_msg):
            self.wizard.get_formats('test.srt.txt')
        with assertRaisesRegex(self, Exception, error_msg):
            self.wizard.get_formats('test.init')
        with assertRaisesRegex(self, Exception, error_msg):
            self.wizard.get_formats('test.txt')

    @patch('txclib.wizard.os.path.isfile')
    def test_run(self, api_mock, isfile_mock):
        isfile_mock.return_value = True

        # list of return values of Api.get for consecutive calls
        self.wizard.api.get.side_effect = [
            {
                "INI": {
                    "mimetype": "text/plain",
                    "file-extensions": ".ini",
                    "description": "Joomla INI File"
                },
                "SRT": {
                    "mimetype": "text/plain",
                    "file-extensions": ".srt",
                    "description": "SubRip subtitles"
                }
            },
            [
                {
                    "name": "Test Org",
                    "slug": "test-org",
                }
            ],
            [
                {
                    "name": "project 1",
                    "source_language": {"code": "en", "name": "English"},
                    "slug": "project-1",
                    "archived": False
                }
            ]
        ]
        with patch('txclib.wizard.input_prompt') as iprompt_mock, \
                patch('txclib.wizard.choice_prompt') as cprompt_mock:

            iprompt_mock.side_effect = ["test_file", "translations/<lang>.txt"]
            cprompt_mock.side_effect = ['INI', 'test-org', "project-1"]
            options = self.wizard.run()

            expected_options = {
                'source_file': 'test_file',
                'expression': 'translations/<lang>.txt',
                'i18n_type': 'INI',
                'source_language': 'en',
                'resource': 'project-1.test-file',
            }
            self.assertDictEqual(options, expected_options)



# -*- coding: utf-8 -*-

"""
Unit tests for processor functions.
"""

import unittest

from txclib.processors import hostname_tld_migration, hostname_ssl_migration


class TestHostname(unittest.TestCase):
    """Test for hostname processors."""

    def test_tld_migration_needed(self):
        """
        Test the tld migration of Transifex, when needed.
        """
        hostnames = [
            'http://transifex.net', 'http://www.transifex.net',
            'https://fedora.transifex.net',
        ]
        for h in hostnames:
            hostname = hostname_tld_migration(h)
            self.assertTrue(hostname.endswith('com'))
        orig_hostname = 'http://www.transifex.net/path/'
        hostname = hostname_tld_migration(orig_hostname)
        self.assertEqual(hostname, orig_hostname.replace('net', 'com', 1))

    def test_tld_migration_needed(self):
        """
        Test that unneeded tld migrations are detected correctly.
        """
        hostnames = [
            'https://www.transifex.com', 'http://fedora.transifex.com',
            'http://www.example.net/path/'
        ]
        for h in hostnames:
            hostname = hostname_tld_migration(h)
            self.assertEqual(hostname, h)

    def test_no_scheme_specified(self):
        """
        Test that, if no scheme has been specified, the https one will be used.
        """
        hostname = '//transifex.net'
        hostname = hostname_ssl_migration(hostname)
        self.assertTrue(hostname.startswith('https://'))

    def test_http_replacement(self):
        """Test the replacement of http with https."""
        hostnames = [
            'http://transifex.com', 'http://transifex.net/http/',
            'http://www.transifex.com/path/'
        ]
        for h in hostnames:
            hostname = hostname_ssl_migration(h)
            self.assertEqual(hostname[:8], 'https://')
            self.assertEqual(hostname[7:], h[6:])

    def test_no_http_replacement_needed(self):
        """Test that http will not be replaces with https, when not needed."""
        for h in ['http://example.com', 'http://example.com/http/']:
            hostname = hostname_ssl_migration(h)
            self.assertEqual(hostname, hostname)



# -*- coding: utf-8 -*-

import os
import unittest
from mock import patch
from txclib.paths import posix_path, native_path


class PosixPathTestcase(unittest.TestCase):
    """Tests for posix paths in various systems."""

    def test_posix_path_in_unix_does_nothing(self):
        orig_path = os.path.abspath(os.getcwd())
        path = posix_path(orig_path)
        self.assertEqual(orig_path, path)

    @patch.object(os, 'sep', new='\\')
    @patch.object(os, 'altsep', new='/')
    def test_posix_path_in_windows_replaces_backslashes(self):
        orig_path = os.path.abspath(os.getcwd())
        path = orig_path.replace('/', '\\')
        self.assertEqual(orig_path, posix_path(path))


class NativePathTestCase(unittest.TestCase):
    """Tests for native paths in various systems."""

    def test_native_path_in_unix_does_nothing(self):
        orig_path = os.path.abspath(os.getcwd())
        path = native_path(orig_path)
        self.assertEqual(orig_path, path)

    @patch.object(os, 'sep', new='\\')
    @patch.object(os, 'altsep', new='/')
    def test_posix_path_in_windows_replaces_backslashes(self):
        orig_path = posix_path(os.path.abspath(os.getcwd()))
        expected_path = orig_path.replace('/', '\\')
        self.assertEqual(expected_path, native_path(orig_path))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from codecs import open
from setuptools import setup

import txclib


def get_file_content(filename):
    with open(filename, "r", encoding="UTF-8") as f:
        return f.read()


setup(
    name="transifex-client",
    version=txclib.__version__,
    entry_points={"console_scripts": ["tx=txclib.cmdline:main"]},
    description="A command line interface for Transifex",
    long_description=get_file_content("README.md"),
    long_description_content_type="text/markdown",
    author="Transifex",
    author_email="admin@transifex.com",
    url="https://www.transifex.com",
    license="GPLv2",
    dependency_links=[],
    setup_requires=[],
    python_requires=">=2.7,!=3.0.*,!=3.1.*,!=3.2.*,!=3.3.*,<3.9",
    install_requires=get_file_content("requirements.txt").splitlines(),
    tests_require=["mock"],
    data_files=[],
    test_suite="tests",
    zip_safe=False,
    packages=["txclib"],
    include_package_data=True,
    package_data={},
    keywords=("translation", "localization", "internationalization",),
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
)
